var Saveable = require('./saveable'),
	_ = require('underscore'),
	Step = require('step'),
	JsUtils = require('../jsUtils');

module.exports = BaseModel = function (objectExtension) {
	// Do some checks for objectExtension validity
	if (!objectExtension) {
		throw new Error('BaseModel should only be created with an extension object');
	} else if (!objectExtension.collection || typeof objectExtension.collection !== 'string') {
		throw new Error('BaseModel extension should have a collection attribute of a string type');
	}
	
	if (!objectExtension.defaults) {
		objectExtension.defaults = {};
	}
	
	// check relation validity
	_.each(objectExtension.relationships, function (value) {
		if (!value.primaryAttr || !value.secondaryAttr || (value.type !== 'parental') || !value.secondaryClass) {
			throw new Error('Badly formed relation: ' + value);
		}
	});
	
	return function (options) {
		// ugly hack to get sub-function callbacks to work
		var that = this;
		
		this.attributes = {};
		this.toJSON = function () {
			return this.attributes;
		};
		this.toResponse = function (res) {
			res.send(this.toJSON());
		};
		
		this.get = function (index) {
			return this.attributes[index];
		};
		
		this.set = function (index, value) {
			if (index[0] === '_') {
				console.log('WARNING: Blocked a model attribute that starts with _. This is potentially VERY dangerous');
			} else {
				this.attributes[index] = value;
			}
		};
		
		this.setAll = function (keyValues) {
			_.each.call(that, keyValues, function (value, index) {
				that.set(index, value);
			});
		};
		
		this.loadRelations = function (callback) {
			var model = this;
			Step(
				function () {
					if (objectExtension.relationships === undefined || objectExtension.relationships.length < 1) {
						this(undefined, []);
					} else {
						var group = this.group();
						_.each(objectExtension.relationships, function (value) {
							if (value.type === 'parental') {
								var search = {};
								search[value.secondaryAttr] = model.get('id');
								
								value.secondaryClass.find(search, function (err, results) {
									if (err) {
										group()(err);
									} else {
										group()(undefined, {
											relationship: value,
											results: results
										});
									}
								});
							}
						});
					}
				},
				function (err, resultSet) {
					if (err) throw err;
					
					_.each(resultSet, function (result, index) {
						if (result.relationship.type === 'parental') {
							var toSet = [];
							_.each(result.results, function (value) {
								toSet.push(value.get('id'));
							});
							model.set(result.relationship.primaryAttr, toSet);
						}
					});
					
					this();
				},
				callback
			);
		};
		
		this.deleteRelations = function (callback) {
			var model = this;
			Step(
				// load objects that are related
				function () {
					if (objectExtension.relationships === undefined || objectExtension.relationships.length < 1) {
						this(undefined, []);
					} else {
						var group = this.group();
						_.each(objectExtension.relationships, function (value) {
							if (value.type === 'parental') {
								var search = {};
								search[value.secondaryAttr] = model.get('id');
								
								value.secondaryClass.find(search, function (err, results) {
									if (err) {
										group()(err);
									} else {
										group()(undefined, {
											relationship: value,
											results: results
										});
									}
								});
							}
						});
					}
				},
				function (err, resultSet) {
					if (err) throw err;
					
					if (resultSet.length < 1) {
						this();
					} else {
						var group = this.group();
						_.each(resultSet, function (result, index) {
							if (result.relationship.type === 'parental') {
								var toSet = [];
								_.each(result.results, function (value) {
									value.delete(group());
								});
							}
						});
					};
				},
				callback
			);
		};
		
		_.extend(this.attributes, objectExtension.defaults, options);
		
		// apply normal BaseModel methods and attributes
		Saveable.apply(this, [objectExtension.collection]);
		
		// grab all custom function from the objectExtension and bring it accross
		for (var property in objectExtension) {
			if (typeof objectExtension[property] === 'function') {
				this[property] = objectExtension[property];
			}
		}
		
		// trigger initialise
		if (this.initialise && typeof this.initialise === 'function') {
			this.initialise(options);
		}
	}
};

